WaxAPIREST.Types.ServerPublicKeyCredentialDescriptor (wax_api_rest v0.5.0)
View SourceSummary
Types
@type flat() :: id :: String.t() | {id :: String.t(), [WaxAPIREST.Types.AuthenticatorTransport.t()]}
A tuple-based representation of t:ServerPublicKeyCredentialDescriptor/0
This representation is designed to make it easier to use it with an external data store such as a database, that has no knowledge of Elixir's data structures
@type t() :: %WaxAPIREST.Types.ServerPublicKeyCredentialDescriptor{ id: String.t(), transports: [WaxAPIREST.Types.AuthenticatorTransport.t()] | nil, type: WaxAPIREST.Types.PublicKeyCredentialType.t() }
Functions
@spec new(String.t(), [WaxAPIREST.Types.AuthenticatorTransport.t()] | nil) :: t()